@@ -9,11 +9,13 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.UUID;
 
 import javax.mail.internet.AddressException;
 import javax.mail.internet.InternetAddress;
 
 import org.apache.commons.lang.NullArgumentException;
+import org.gcube.portal.databook.shared.Attachment;
 import org.gcube.portal.databook.shared.Comment;
 import org.gcube.portal.databook.shared.Feed;
 import org.gcube.portal.databook.shared.FeedType;
@@ -82,7 +84,9 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 	public static final String HASHTAGS_COUNTER = "HashtagsCounter"; // count the hashtags per group and type
 	public static final String HASHTAGGED_FEEDS = "HashtaggedFeeds"; // contains hashtags per type associated with vre and feed
 	public static final String VRE_INVITES = "VREInvites"; //contains the emails that were invited per VRE
-	public static final String EMAIL_INVITES = "EMAILInvites"; //contains the list of invitation per email
+	public static final String EMAIL_INVITES = "EMAILInvites"; //contains the list of invitation per email	
+	public static final String ATTACHMENTS = "Attachments"; //contains the list of all the attachments in a feed
+	public static final String FEED_ATTACHMENTS = "FeedAttachments"; //contains the list of all the attachments for a given feed (dynamic CF)
 
 
 	private static ColumnFamily<String, String> cf_Connections = new ColumnFamily<String, String>(
@@ -162,7 +166,10 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 			StringSerializer.get(),   // Key Serializer
 			StringSerializer.get());  // Column Serializer
 
-
+	protected static ColumnFamily<String, String> cf_Attachments =	new ColumnFamily<String, String>(
+			ATTACHMENTS,  // Column Family Name
+			StringSerializer.get(),   // Key Serializer
+			StringSerializer.get());  // Column Serializer
 
 	/**
 	 * connection instance
@@ -331,7 +338,8 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 		.putColumn("LinkTitle", feed.getLinkTitle(), null)
 		.putColumn("LinkDescription", feed.getLinkDescription(), null)
 		.putColumn("LinkHost", feed.getLinkHost(), null)
-		.putColumn("IsApplicationFeed", feed.isApplicationFeed(), null);
+		.putColumn("IsApplicationFeed", feed.isApplicationFeed(), null)
+		.putColumn("multiFileUpload", feed.isMultiFileUpload(), null);
 		return m;
 	}
 	/**
@@ -353,6 +361,26 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 		}
 		return execute(m);
 	}
+	/**
+	 * {@inheritDoc}
+	 */
+	@Override
+	public boolean saveUserFeed(Feed feed, List<Attachment> attachments) {
+		if (attachments != null && !attachments.isEmpty())
+			feed.setMultiFileUpload(true);
+		boolean saveFeedResult = saveUserFeed(feed);
+		if (saveFeedResult) {
+			String feedKey = feed.getKey();
+			for (Attachment attachment : attachments) {
+				boolean attachSaveResult = saveAttachmentEntry(feedKey, attachment);
+				if (!attachSaveResult)
+					_log.warn("Some of the attachments failed to me saved: " + attachment.getName());
+			}
+			return true;
+		}
+		else return false;
+	}
+	
 	/**
 	 * {@inheritDoc}
 	 */
@@ -379,6 +407,25 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 	 * {@inheritDoc}
 	 */
 	@Override
+	public boolean saveAppFeed(Feed feed, List<Attachment> attachments) {
+		if (attachments != null && !attachments.isEmpty())
+			feed.setMultiFileUpload(true);
+		boolean saveFeedResult = saveAppFeed(feed);
+		if (saveFeedResult) {
+			String feedKey = feed.getKey();
+			for (Attachment attachment : attachments) {
+				boolean attachSaveResult = saveAttachmentEntry(feedKey, attachment);
+				if (!attachSaveResult)
+					_log.warn("Some of the attachments failed to me saved: " + attachment.getName());
+			}
+			return true;
+		}
+		else return false;
+	}
+	/**
+	 * {@inheritDoc}
+	 */
+	@Override
 	public boolean saveFeedToVRETimeline(String feedKey, String vreid) throws FeedIDNotFoundException {
 		String feedId = feedKey;
 		Feed toCheck = null;
@@ -438,6 +485,12 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 			toReturn.setLinkDescription(columns.getColumnByName("LinkDescription").getStringValue());
 			toReturn.setLinkHost(columns.getColumnByName("LinkHost").getStringValue());
 			toReturn.setApplicationFeed(columns.getColumnByName("IsApplicationFeed").getBooleanValue());
+			boolean isMultiFileUpload = false;
+			try {
+				isMultiFileUpload = columns.getColumnByName("multiFileUpload").getBooleanValue();
+			}
+			catch (NullPointerException e) { }
+			toReturn.setMultiFileUpload(isMultiFileUpload);
 
 		} catch (ConnectionException e) {
 			e.printStackTrace();
@@ -449,6 +502,44 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 	 * {@inheritDoc}
 	 */
 	@Override
+	public List<Feed> getRecentFeedsByUserAndDate(String userid, long timeInMillis) throws IllegalArgumentException {
+		Date now = new Date();
+		if (timeInMillis > now.getTime())
+			throw new IllegalArgumentException("the timeInMillis must be before today");
+
+		OperationResult<Rows<String, String>> result = null;
+		try {
+			result = conn.getKeyspace().prepareQuery(cf_UserTline)
+					.getKeySlice(userid)
+					.execute();
+		} catch (ConnectionException e) {
+			e.printStackTrace();
+		}
+		List<Feed> toReturn = new ArrayList<Feed>();
+		// Iterate rows and their columns 
+		for (Row<String, String> row : result.getResult()) {
+			for (Column<String> column : row.getColumns()) {
+				long feedTime = Long.parseLong(column.getName());
+				if (feedTime > timeInMillis) {
+					try {
+						Feed toCheck = readFeed(column.getStringValue());
+						if (toCheck.getType() != FeedType.DISABLED)
+							toReturn.add(toCheck);
+					} catch (PrivacyLevelTypeNotFoundException
+							| FeedTypeNotFoundException
+							| FeedIDNotFoundException
+							| ColumnNameNotFoundException e) {
+						e.printStackTrace();
+					}
+				}
+			}
+		}
+		return toReturn;
+	}
+	/**
+	 * {@inheritDoc}
+	 */
+	@Override
 	public boolean deleteFeed(String feedId) throws FeedIDNotFoundException, PrivacyLevelTypeNotFoundException, FeedTypeNotFoundException, ColumnNameNotFoundException {
 		Feed toDelete = readFeed(feedId);
 		MutationBatch m = conn.getKeyspace().prepareMutationBatch();
@@ -576,6 +667,7 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 	/**
 	 * {@inheritDoc}
 	 */
+	@SuppressWarnings("deprecation")
 	@Override
 	public List<Feed> getAllPortalPrivacyLevelFeeds() throws FeedTypeNotFoundException, ColumnNameNotFoundException, PrivacyLevelTypeNotFoundException {
 		ArrayList<Feed> toReturn = new ArrayList<Feed>();
@@ -1230,8 +1322,8 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 		.putColumn("Userid", comment.getUserid(), null)
 		.putColumn("Feedid",comment.getFeedid(), null)
 		.putColumn("FullName",comment.getFullName(), null)
-		.putColumn("ThumbnailURL", comment.getThumbnailURL(), null);
-
+		.putColumn("ThumbnailURL", comment.getThumbnailURL(), null)
+		.putColumn("IsEdited", comment.isEdit(), null);
 
 		try {
 			m.execute();			
@@ -1247,7 +1339,6 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 	 * {@inheritDoc}
 	 */
 	@Override
-	@SuppressWarnings("unchecked")
 	public List<Comment> getAllCommentByFeed(String feedid) {
 		List<Comment> toReturn = new ArrayList<Comment>();
 
@@ -1277,6 +1368,10 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 						toAdd.setThumbnailURL(col.getStringValue());
 					else if (col.getName().compareTo("Feedid") == 0)
 						toAdd.setFeedid(col.getStringValue());
+					else if(col.getName().compareTo("IsEdited") == 0)
+						toAdd.setEdit(col.getBooleanValue());
+					else if(col.getName().compareTo("LastEditTime") == 0)
+						toAdd.setLastEditTime(getDateFromTimeInMillis(col.getStringValue()));
 					else {
 						_log.error("getAllCommentByFeed(): Could not assign variable to this Comment for column name: " + col.getName());
 					}
@@ -1289,6 +1384,7 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 		}
 		return toReturn;
 	}
+
 	/**
 	 * {@inheritDoc}
 	 */
@@ -1297,7 +1393,8 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 		MutationBatch m = conn.getKeyspace().prepareMutationBatch();
 		//an entry in the feed CF
 		m.withRow(cf_Comments, comment2Edit.getKey().toString()).putColumn("Text", comment2Edit.getText(), null);
-		m.withRow(cf_Comments, comment2Edit.getKey().toString()).putColumn("Timestamp", comment2Edit.getTime().getTime()+"", null);
+		m.withRow(cf_Comments, comment2Edit.getKey().toString()).putColumn("IsEdited", comment2Edit.isEdit(), null);
+		m.withRow(cf_Comments, comment2Edit.getKey().toString()).putColumn("LastEditTime", comment2Edit.getLastEditTime().getTime() + "", null);
 		try {
 			m.execute();
 		} catch (ConnectionException e) {
@@ -1307,6 +1404,7 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 		_log.info("Comments update OK to: " + comment2Edit.getText());
 		return true;
 	}
+
 	/**
 	 * {@inheritDoc}
 	 */
@@ -1606,8 +1704,8 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 	 *
 	 */
 	/**
-	 * common part to save a feed
-	 * @param feed
+	 * common part to save a invite
+	 * @param invite
 	 * @return the partial mutation batch instance
 	 */
 	private MutationBatch initSaveInvite(Invite invite) {
@@ -1615,7 +1713,7 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 			throw new NullArgumentException("Invite instance is null");
 		// Inserting data
 		MutationBatch m = conn.getKeyspace().prepareMutationBatch();
-		//an entry in the feed CF
+		//an entry in the invite CF
 		m.withRow(cf_Invites, invite.getKey().toString())
 		.putColumn("SenderUserId", invite.getSenderUserId(), null)
 		.putColumn("Vreid", invite.getVreid(), null)
@@ -1785,6 +1883,60 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 	 * {@inheritDoc}
 	 */
 	@Override
+	public List<Attachment> getAttachmentsByFeedId(String feedId) throws FeedIDNotFoundException {
+		Feed toCheck = null;
+		try {
+			toCheck = readFeed(feedId);
+			if (toCheck == null)
+				throw new FeedIDNotFoundException("Could not find Feed with id " + feedId);
+		} catch (Exception e) {
+			e.printStackTrace();
+			return null;
+		} 
+		List<Attachment> toReturn = new ArrayList<Attachment>();
+
+		PreparedIndexExpression<String, String> clause = cf_Attachments.newIndexClause().whereColumn("feedId").equals().value(feedId);
+		OperationResult<Rows<String, String>> result;
+		try {
+			result = conn.getKeyspace().prepareQuery(cf_Attachments)
+					.searchWithIndex()
+					.setStartKey("")
+					.addPreparedExpressions(Arrays.asList(clause))
+					.execute();
+
+			// Iterate rows and their columns 
+			for (Row<String, String> row : result.getResult()) {
+				Attachment toAdd = new Attachment();
+				toAdd.setId(row.getKey());
+				for (Column<String> col : row.getColumns()) {
+					if (col.getName().compareTo("feedId") == 0)
+						_log.trace("Reading attachment if feed=" + col.getStringValue());
+					else if (col.getName().compareTo("uri") == 0)
+						toAdd.setUri(col.getStringValue());
+					else if (col.getName().compareTo("name") == 0) 
+						toAdd.setName(col.getStringValue());
+					else if (col.getName().compareTo("description") == 0)
+						toAdd.setDescription(col.getStringValue());
+					else if (col.getName().compareTo("ThumbnailURL") == 0)
+						toAdd.setThumbnailURL(col.getStringValue());
+					else if (col.getName().compareTo("mimeType") == 0)
+						toAdd.setMimeType(col.getStringValue());
+					else {
+						_log.error("getAttachmentsByFeedId(): Could not assign variable to this Attachment for column name: " + col.getName());
+					}
+				}
+				toReturn.add(toAdd);
+			}
+		} catch (Exception e) {
+			e.printStackTrace();
+			return null;
+		}
+		return toReturn;
+	}
+	/**
+	 * {@inheritDoc}
+	 */
+	@Override
 	public void closeConnection() {
 	}
 	/*
@@ -1792,6 +1944,31 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 	 ********************** 	Helper methods	***********************
 	 *
 	 */
+	/**
+	 * @param feedId the feedId to which the attachment is attached
+	 * @param toSave the instance to save
+	 * @return true if the attachemnt entry is saved in the Attachments CF
+	 */
+	private boolean saveAttachmentEntry(String feedId, Attachment toSave) {
+		// Inserting data
+		MutationBatch m = conn.getKeyspace().prepareMutationBatch();
+		//an entry in the Attachment CF
+		m.withRow(cf_Attachments, toSave.getId())
+		.putColumn("feedId", feedId, null)
+		.putColumn("uri", toSave.getUri(), null)
+		.putColumn("name", toSave.getName(), null)
+		.putColumn("description",toSave.getDescription(), null)
+		.putColumn("thumbnailURL",toSave.getThumbnailURL(), null)
+		.putColumn("mimeType",toSave.getMimeType(), null);
+		try {
+			m.execute();			
+		} catch (ConnectionException e) {
+			e.printStackTrace();
+			return false;
+		}
+		return true;
+	}
+
 
 	/**
 	 * simply return an enum representing the privacy level
@@ -2130,4 +2307,6 @@ public final class DBCassandraAstyanaxImpl implements DatabookStore {
 
 
 
+
+
 }
